package geometrie;

import java.io.*;
import java.math.*;

/**
 * classe Angles3
 * classe reprsentant les angles theta, phi et psi ncessaire au positionnement et  l'orientation 3D
 */
 
public class Angles3 {
    
    private double theta, phi, psi; //angles
    
   /**
    * constructeur Angles3
    * construit un Angles3 avec 0 comme valeurs par dfaut
    */
    public Angles3() {
        theta=0; phi=0; psi=0;
    }
    
   /**
    * constructeur Angles
    * construit un Angles avec les paramtres spcifis
    * @param theta_ l'angle theta
    * @param phi_   l'angle phi
    * @param psi_   l'angle psi
    */
    public Angles3(double theta_, double phi_, double psi_) {
        theta=theta_; phi=phi_; psi=psi_;
    }
    
   /**
    * methode get_theta
    * retourne l'angle theta
    * @return l'angle theta
    */
    public double get_theta() {return theta;}

   /**
    * methode get_phi
    * retourne l'angle phi
    * @return l'angle phi
    */
    public double get_phi() {return phi;}

   /**
    * methode get_psi
    * retourne l'angle psi
    * @return l'angle psi
    */
    public double get_psi() {return psi;}
    
   /**
    * methode set_theta
    * permet de modifier l'angle theta
    * @param v l'angle theta
    */
    public void set_theta(double v) {theta=v;}

   /**
    * methode set_phi
    * permet de modifier l'angle phi
    * @param v l'angle phi
    */
    public void set_phi(double v) {phi=v;}

   /**
    * methode set_psi
    * permet de modifier l'angle psi
    * @param v l'angle psi
    */
    public void set_psi(double v) {psi=v;}

   /**
    * methode random
    * gnre des angles alatoires pour theta ( rpartition de cos(theta) uniforme ) et phi (rpartition uniforme), soit une gnration alatoire sur une sphre. Gnration uniforme pour psi galement
    */
    public void random() {
        theta = Math.acos(Math.random()*2-1);
        phi = Math.random()*2*Math.PI;
        psi = Math.random()*2*Math.PI;
    }

   /**
    * methode random_plate
    * gnre des angles alatoires psi et phi, et une gnration alatoire de theta autour de 0
    * @param tilt amplitude angulaire en degrs selon theta, suivant une loi en sin(rnd-0.5)
    */
    public void random_plate(double tilt) {
        theta = tilt/180*Math.PI*Math.sin(Math.random()-0.5);
        phi = Math.random()*2*Math.PI;
        psi = Math.random()*2*Math.PI;
    }

   /**
    * methode random_column
    * gnre des angles alatoires psi et phi, et une gnration alatoire de theta autour de 90
    * @param tilt amplitude angulaire en degrs selon theta, suivant une loi en sin(rnd-0.5)
    */
    public void random_column(double tilt) {
        theta = Math.PI/2+tilt/180*Math.PI*Math.sin(Math.random()-0.5);
        phi = Math.random()*2*Math.PI;
        psi = Math.random()*2*Math.PI;
    }

   /**
    * methode random_column2
    * gnre des angles alatoires psi, une gnration alatoire de theta autour de 90 et de psi autour de 30
    * @param tilt amplitude angulaire en degrs selon theta, suivant une loi en sin(rnd-0.5)
    */
    public void random_column2(double tilt) {
        theta = Math.PI/2+tilt/180*Math.PI*Math.sin(Math.random()-0.5);
        phi = Math.random()*2*Math.PI;
        psi = Math.PI/6+tilt*(Math.random()-0.5);
    }

   /**
    * methode affiche
    * affiche les angles theta, phi et psi
    */
    public void affiche() {
        System.out.println(theta+" "+phi+" "+psi);
    }

   /**
    * methode print
    * enregistre les angles theta, phi et psi dans le flux spcifi
    * @param out flux de sortie
    */
    public void print(PrintWriter out) {
        out.print(theta); out.print(" "); out.print(phi); out.print(" "); out.print(psi); out.println();
    }

}
