package geometrie;

import java.io.*;
import java.math.*;

/**
 * classe Droite
 * classe reprsentant une droite dans l'espace
 */
 
public class Droite {
    
    private Vecteur v;   //le vecteur directeur
    private Point P;     //un point de la droite
    
   /**
    * constructeur Droite 
    * construit une Droite avec des paramtres nuls par dfaut
    */
    public Droite() {
        v = new Vecteur();
        P = new Point();
    }
    
   /**
    * constructeur Droite 
    * construit une Droite avec les paramtres spcifies
    * @param v_ le vecteur directeur
    * @param P_ un point de la droite
    */
    public Droite(Vecteur v_, Point P_) {
        v=v_; P=P_;
    }
    
   /**
    * methode get_v
    * retourne le vecteur directeur
    * @return le vecteur directeur
    */
    public Vecteur get_v() {return v;}

   /**
    * methode get_P
    * retourne le point de rfrence de la droite
    * @return le point de rfrence de la droite
    */
    public Point get_P() {return P;}

   /**
    * methode set_v
    * permet de modifier le vecteur directeur
    * @param v_ le vecteur directeur
    */
    public void set_v(Vecteur v_) {v=v_;}

   /**
    * methode set_P
    * permet de modifier le point de rfrence
    * @param P_ le point de rfrence
    */
    public void set_P(Point P_) {P=P_;}

    /**
    * methode set_P
    * permet de modifier le point de rfrence
    * @param x_ la coordonne x
    * @param y_ la coordonne y
    * @param z_ la coordonne z
    */
    public void set_P(double x_, double y_, double z_) {P.set(x_, y_, z_);}
    
   /**
    * methode set
    * permet de modifier les coordonnes de la droite
    * @param v_ le vecteur directeur
    * @param P_ le point de rfrence
    */
    public void set(Vecteur v_, Point P_) {
        v=v_; P=P_;
    }

    /**
    * methode rotated_x
    * tourne la droite d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    */
    public void rotated_x(double angle) {
        v.rotated_x(angle);
        P.rotated_x(angle);
    }

   /**
    * methode rotated_y
    * tourne la droite d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    */
    public void rotated_y(double angle) {
        v.rotated_y(angle);
        P.rotated_y(angle);
    }

   /**
    * methode rotated_z
    * tourne la droite d'un angle donn autour de l'axe z
    * @param angle l'angle de rotation
    */
    public void rotated_z(double angle) {
        v.rotated_z(angle);
        P.rotated_z(angle);
    }

   /**
    * methode affiche
    * affiche les coordonnes
    */
    public void affiche() {
        System.out.println("Droite :"); v.affiche(); P.affiche();
    }

   /**
    * methode print
    * enregistre les coordonnes de deux points matrialisant la droite dans le flux spcifi
    * @param out flux de sortie
    * @param d1  la distance au point de rfrence o commencer l'affichage de la droite
    * @param d2  la distance au point de rfrence o finir l'affichage de la droite
    */
    public void print(PrintWriter out, double d1, double d2) {
        P.translate(v.fois(d1)).print(out);
        P.translate(v.fois(d2)).print(out);
        out.println();
        out.println();
    }

   /**
    * methode print
    * enregistre les coordonnes de deux points, dont le point de rfrence, matrialisant la droite dans le flux spcifi
    * @param out flux de sortie
    * @param d   la distance au point de rfrence o finir l'affichage de la droite
    */
    public void print(PrintWriter out, double d) {
        P.print(out);
        P.translate(v.fois(d)).print(out);
        out.println();
        out.println();
    }
}
