package geometrie;

import java.awt.*;

/**
 * classe Forme3D
 * classe abstraite reprsentant une forme gomtrique en 3D
 */
 
abstract public class Forme3D extends FormeGeometrique {

   /**
    * constructeur Forme3D 
    * consrtruit un objet abstrait d'une forme gomtrique en 3D
    */
    public Forme3D() {
    
    }

   /**
    * methode distance_intersect
    * retourne la distance entre le point source d'un rayon lumineux et son point d'intersection avec la forme 3D
    * @param r le rayon lumineux
    * @return la distance entre le point source du rayon lumineux et son point d'intersection avec la forme 3D
    */
    //abstract public double distance_intersect(Rayon r);
    
   /**
    * methode distance_intersect
    * retourne la distance entre le point de rfrence de la droite et son point d'intersection avec la forme 3D
    * @param d la droite
    * @return la distance entre le point de rfrence de la droite et son point d'intersection avec la forme 3D
    */
    abstract public double distance_intersect(Droite d);

   /**
    * methode intersect
    * retourne le point d'intersection d'une droite avec la forme 3D
    * @param d la droite
    * @return le point d'intersection de la droite avec la forme 3D
    */
    abstract public Point intersect(Droite d);

   /**
    * methode rotated_x
    * tourne la forme 3D d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    */
    abstract public void rotated_x(double angle);

   /**
    * methode rotated_y
    * tourne la forme 3D d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    */
    abstract public void rotated_y(double angle);

   /**
    * methode get_normale
    * retourne la normale  la surface au point demand
    * @param Pt le point o calculer la normale
    * @return la normale  la surface au point demand
    */
    abstract public Vecteur get_normale(Point Pt);
   
   /**
    * methode in_projection_z
    * retourne si le point P est inclu dans la forme gomtrique projete selon z
    * @return un booleen indiquant si le point P est inclu dans la forme gomtrique  projete selon z
    */
    abstract public boolean in_projection_z(Point2D P);
    
   /**
    * methode distance_max
    * retourne le rayon de la sphre, centre sur le point donn, circonscrite  la forme gomtrique
    * @param P le point de rfrence
    * @return le rayon de la sphre, centre sur le point donn, circonscrite  la forme gomtrique
    */
    abstract public double distance_max(Point P);

   /**
    * methode draw
    * retourne l'objet graphique pass en argument aprs ajout de la reprsentation de la forme gomtrique
    * @param g       l'objet graphique
    * @param facteur le facteur de grandissement  appliquer
    * @return l'objet graphique aprs ajout de la reprsentation de la forme gomtrique
    */    
    abstract public Graphics draw(Graphics g, double facteur);
}