package geometrie;

/**
 * classe Plan
 * classe abstraite reprsentant un plan dans l'espace
 */
 
abstract public class Plan extends Forme3D {

    protected Vecteur n;        //vecteur normal au plan
    protected Point P;          //point appartenant au plan
    
   /**
    * constructeur Plan 
    * construit un objet abstrait plan avec les paramtres spcifis
    * @param P_         un point du plan
    * @param n_         la normale au plan
    */
    public Plan( Point P_, Vecteur n_) {
        P=P_;
        n=n_.normalised();
    }
    
   /**
    * methode get_n
    * retourne le vecteur normal au plan
    * @return le vecteur normal au plan
    */
    public Vecteur get_n() {return n;}

   /**
    * methode get_P
    * retourne le point de rfrence du plan
    * @return le point de rfrence du plan
    */
    public Point get_P() {return P;}

   /**
    * methode get_normale
    * retourne la normale  la surface au point demand
    * @param Pt le point o calculer la normale, indiffrent ici
    * @return le vecteur normal au plan
    */
    public Vecteur get_normale(Point Pt) {return n;}

   /**
    * methode rotated_x
    * tourne la forme 3D d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    */
    public void rotated_x(double angle) {
        P.rotated_x(angle);
        n.rotated_x(angle);
    }

   /**
    * methode rotated_y
    * tourne la forme 3D d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    */
    public void rotated_y(double angle) {
        P.rotated_y(angle);
        n.rotated_y(angle);
    }

   /**
    * methode rotated_z
    * tourne la forme 3D d'un angle donn autour de l'axe z
    * @param angle l'angle de rotation
    */
    public void rotated_z(double angle) {
        P.rotated_z(angle);
        n.rotated_z(angle);
    }

   /**
    * methode intersect
    * retourne le point d'intersection d'une droite avec la forme 3D
    * @param d la droite
    * @return le point d'intersection de la droite avec la forme 3D
    */
    public Point intersect(Droite d) {
        double lambda = -n.scalaire(P.vecteur(d.get_P())) / n.scalaire(d.get_v());
        Point I = (d.get_P()).translate((d.get_v()).fois(lambda));
        return I;
    }
    
   /**
    * methode distance_intersect
    * retourne la distance entre le point de rfrence de la droite et son point d'intersection avec la forme 3D
    * @param d la droite
    * @return la distance entre le point de rfrence de la droite et son point d'intersection avec la forme 3D
    */
    public double distance_intersect(Droite d) {
        return -n.scalaire(P.vecteur(d.get_P())) / n.scalaire(d.get_v());
    }

}