package geometrie;

import java.io.*;
import java.math.*;

/**
 * classe Point
 * classe reprsentant un point dans l'espace
 */
 
public class Point {
    
    private double x,y,z; //les coordonnes du point
    
   /**
    * constructeur Point 
    * construit un Point avec 0 comme coordonnes par dfaut
    */
    public Point() {
        x=0; y=0; z=0;
    }
    
   /**
    * constructeur Point 
    * construit un Point avec les coordonnes spcifies
    * @param x_ la coordonne x
    * @param y_ la coordonne y
    * @param z_ la coordonne z
    */
    public Point(double x_, double y_, double z_) {
        x=x_; y=y_; z=z_;
    }
    
   /**
    * methode get_x
    * retourne la valeur de la coordonne x
    * @return la valeur de la coordonne x
    */
    public double get_x() {return x;}

   /**
    * methode get_y
    * retourne la valeur de la coordonne y
    * @return la valeur de la coordonne y
    */
    public double get_y() {return y;}

   /**
    * methode get_z
    * retourne la valeur de la coordonne z
    * @return la valeur de la coordonne z
    */
    public double get_z() {return z;}
    
   /**
    * methode set_x
    * permet de modifier la coordonne x
    * @param v la valeur de la coordonne x
    */
    public void set_x(double v) {x=v;}

   /**
    * methode set_y
    * permet de modifier la coordonne y
    * @param v la valeur de la coordonne y 
    */
    public void set_y(double v) {y=v;}

   /**
    * methode set_z
    * permet de modifier la coordonne z
    * @param v la valeur de la coordonne z
    */
    public void set_z(double v) {z=v;}
    
   /**
    * methode set
    * permet de modifier les coordonnes du point
    * @param x_ la valeur de la coordonne x
    * @param y_ la valeur de la coordonne y
    * @param z_ la valeur de la coordonne z
    */
    public void set(double x_, double y_, double z_) {
        x=x_; y=y_; z=z_;
    }

   /**
    * methode translate
    * retourne le point translat d'un vecteur
    * @param v vecteur dont translater le point
    * @return le point translat
    */
    public Point translate(Vecteur v) {
        return new Point(x+v.get_x(), y+v.get_y(), z+v.get_z());
    }

   /**
    * methode translated
    * translate le point d'un vecteur
    * @param v vecteur dont translater le point
    */
    public void translated(Vecteur v) {
        x+=v.get_x();
        y+=v.get_y();
        z+=v.get_z();
    }

   /**
    * methode rotate_x
    * retourne le point aprs rotation d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    * @return le point aprs rotation
    */
    public Point rotate_x(double angle) {
        return new Point(x, y*Math.cos(angle)-z*Math.sin(angle), z*Math.cos(angle)+y*Math.sin(angle));
    }

   /**
    * methode rotate_y
    * retourne le point aprs rotation d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    * @return le point aprs rotation
    */
    public Point rotate_y(double angle) {
        return new Point(x*Math.cos(angle)+z*Math.sin(angle), y, z*Math.cos(angle)-x*Math.sin(angle));
    }

   /**
    * methode rotate_z
    * retourne le point aprs rotation d'un angle donn autour de l'axe z
    * @param angle l'angle de rotation
    * @return le point aprs rotation
    */
    public Point rotate_z(double angle) {
        return new Point(x*Math.cos(angle)-y*Math.sin(angle), y*Math.cos(angle)+x*Math.sin(angle), z);
    }

   /**
    * methode rotated_x
    * tourne le point d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    */
    public void rotated_x(double angle) {
        double Y = y;
        y=Y*Math.cos(angle)-z*Math.sin(angle);
        z=z*Math.cos(angle)+Y*Math.sin(angle);
    }

   /**
    * methode rotated_y
    * tourne le point d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    */
    public void rotated_y(double angle) {
        double Z = z;
        z=Z*Math.cos(angle)-x*Math.sin(angle);
        x=x*Math.cos(angle)+Z*Math.sin(angle);
    }

   /**
    * methode rotated_z
    * tourne le point d'un angle donn autour de l'axe z
    * @param angle l'angle de rotation
    */
    public void rotated_z(double angle) {
        double X = x;
        x=X*Math.cos(angle)-y*Math.sin(angle);
        y=y*Math.cos(angle)+X*Math.sin(angle);
    }
    
   /**
    * methode tourne
    * tourne le point des angles theta, phi, psi indiqu par rapport  sa position prcdente
    * @param dtheta l'angle theta
    * @param dphi   l'angle phi
    * @param dpsi   l'angle psi
    */
    public void tourne(double dtheta, double dphi, double dpsi) {
        rotated_z(dpsi);
        rotated_y(dtheta);
        rotated_z(dphi);
    }

   /**
    * methode distance
    * retourne la distance  un autre point
    * @param B l'autre point
    * @return la distance  l'autre point
    */
    public double distance(Point B) {
        return Math.sqrt((x-B.get_x())*(x-B.get_x())+(y-B.get_y())*(y-B.get_y())+(z-B.get_z())*(z-B.get_z()));
    }

   /**
    * methode vecteur
    * retourne le vecteur form par ce point et celui donn en argument
    * @param B le point formant le bout du vecteur
    * @return le vecteur form par ce point et celui donn en argument
    */
    public Vecteur vecteur(Point B) {
        return new Vecteur(B.get_x()-x, B.get_y()-y, B.get_z()-z);
    }
    
   /**
    * methode affiche
    * affiche les coordonnes
    */
    public void affiche() {
        System.out.println(x+" "+y+" "+z);
    }

   /**
    * methode print
    * enregistre les coordonnes dans le flux spcifi
    * @param out flux de sortie
    */
    public void print(PrintWriter out) {
        out.print(x); out.print(" "); out.print(y); out.print(" "); out.print(z); out.println();
    }

}
