package geometrie;

import java.io.*;
import java.awt.*;

/**
 * classe Polygone
 * classe reprsentant un polygone dans l'espace
 */
 
public class Polygone extends Plan {

    protected Point[] coins; //liste des coordonnes des sommets du polygone
    
   /**
    * constructeur Polygone 
    * construit un polygone plan avec les paramtres spcifis
    * @param centre     le centre du polygone (pour les rotations), appartenant au plan o celui-ci est dfini
    * @param n_         le vecteur normal au polygone
    */
    public Polygone(Point centre, Vecteur n_) {
        super(centre, n_);
    }
    
   /**
    * methode get_coin
    * retourne le coin voulu du polygone
    * @param i l'indice du coin voulu
    * @return le i-me coin du polygone
    */
    public Point get_coin(int i) {
        return coins[i];
    }

   /**
    * methode rotated_x
    * tourne le polygone d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    */
    public void rotated_x(double angle) {
        for (int i=0; i<coins.length; i++) coins[i].rotated_x(angle);
        super.rotated_x(angle);
    }

   /**
    * methode rotated_y
    * tourne le polygone d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    */
    public void rotated_y(double angle) {
        for (int i=0; i<coins.length; i++) coins[i].rotated_y(angle);
        super.rotated_y(angle);
    }

   /**
    * methode rotated_z
    * tourne le polygone d'un angle donn autour de l'axe z
    * @param angle l'angle de rotation
    */
    public void rotated_z(double angle) {
        for (int i=0; i<coins.length; i++) coins[i].rotated_z(angle);
        super.rotated_z(angle);
    }
    
   /**
    * methode affiche
    * affiche les coordonnes des coins du polygone
    */
    public void affiche() {
        System.out.println("Liste des points :");
        for(int i=0; i<coins.length; i++) coins[i].affiche();
        System.out.println("vecteur normal :");
        n.affiche();
    }

   /**
    * methode print
    * enregistre les coordonnes des coins du polygone dans le flux spcifi
    * @param out flux de sortie
    */
    public void print(PrintWriter out) {
        for(int i=0; i<coins.length; i++) (coins[i]).print(out);
        (coins[0]).print(out);
        out.println(); out.println();
    }

   /**
    * methode in_projection_z
    * retourne si le point P est inclu dans la forme gomtrique projete selon z
    * @return un booleen indiquant si le point P est inclu dans la forme gomtrique  projete selon z
    */
    public boolean in_projection_z(Point2D P) {
        int l = coins.length;
        double xp = P.get_x();
        double yp = P.get_y();
        double xi = coins[0].get_x();
        double yi = coins[0].get_y();
        double xi1 = coins[1].get_x();
        double yi1 = coins[1].get_y();
        int sgn = (int)signe( (xi1-xi)*(yp-yi) - (yi1-yi)*(xp-xi) );
        for (int i=1; i<l; i++) {
            xi = xi1;
            yi = yi1;
            xi1 = coins[(i+1)%l].get_x();
            yi1 = coins[(i+1)%l].get_y();
            if ( (int)signe( (xi1-xi)*(yp-yi) - (yi1-yi)*(xp-xi) ) != sgn) return false;
        }
        return true;
    }
    
   /**
    * methode distance_max
    * retourne le rayon de la sphre, centre sur le point donn, circonscrite  la forme gomtrique
    * @param P le point de rfrence
    * @return le rayon de la sphre, centre sur le point donn, circonscrite  la forme gomtrique
    */
    public double distance_max(Point P) {
        double dmax=0, d;
        for (int i=0; i<coins.length; i++) {
            d=P.distance(coins[i]);
            if (d>dmax) dmax=d;
        }
        return dmax;
    }
    
   /**
    * methode draw
    * retourne l'objet graphique pass en argument aprs ajout de la reprsentation de la forme gomtrique
    * @param g       l'objet graphique
    * @param facteur le facteur de grandissement  appliquer
    * @return l'objet graphique aprs ajout de la reprsentation de la forme gomtrique
    */    
    public Graphics draw(Graphics g, double facteur) {
        int l = coins.length;
        for (int i=0; i<l; i++) g.drawLine((int)Math.floor(coins[i].get_y()*facteur), (int)Math.floor(coins[i].get_z()*facteur), (int)Math.floor(coins[(i+1)%l].get_y()*facteur), (int)Math.floor(coins[(i+1)%l].get_z()*facteur));
        return g;
    }

}
