package geometrie;

import java.io.*;
import java.awt.*;

/**
 * classe Sphere
 * classe reprsentant une sphre dans l'espace
 */
 
public class Sphere extends Forme3D {

    private double R;           //rayon de la sphre
    private Point O;            //centre de la sphre
    
   /**
    * constructeur Sphere 
    * construit une sphre avec les paramtres spcifis
    * @param O_         le centre de la sphre
    * @param R_         le rayon de la sphre
    */
    public Sphere(Point O_, double R_) {
        R=R_;
        O=O_;
    }
    
   /**
    * methode get_normale
    * retourne la normale  la surface au point demand
    * @param Pt le point o calculer la normale
    * @return la normale  la surface au point demand
    */
    public Vecteur get_normale(Point Pt) {
        return O.vecteur(Pt).normalised();
    }

   /**
    * methode intersect
    * retourne le point d'intersection d'une droite avec la forme 3D
    * @param d la droite
    * @return le point d'intersection de la droite avec la forme 3D
    */
    public Point intersect(Droite d) {
        double VCD = (d.get_v()).scalaire(O.vecteur(d.get_P()));
        double x = Math.sqrt(VCD*VCD+R*R-(O.vecteur(d.get_P())).norme_carre());
        double lambda;
        if (-x-VCD>1e-5) lambda = -x-VCD; else lambda = x-VCD;
        Point I = (d.get_P()).translate((d.get_v()).fois(lambda));
        return I;
    }

   /**
    * methode distance_intersect
    * retourne la distance entre le point de rfrence de la droite et son point d'intersection avec la forme 3D
    * @param d la droite
    * @return la distance entre le point de rfrence de la droite et son point d'intersection avec la forme 3D
    */
    public double distance_intersect(Droite d) {
        double VCD = (d.get_v()).scalaire(O.vecteur(d.get_P()));
        double x = Math.sqrt(VCD*VCD+R*R-(O.vecteur(d.get_P())).norme_carre());
        //System.out.println("x:"+x+"  VCD:"+VCD);
        if (-x-VCD>1e-5) return -x-VCD; else return x-VCD;
    }
    
   /**
    * methode affiche
    * affiche les caractristiques de la sphre
    */
    public void affiche() {
        System.out.println("Centre : "); O.affiche();
        System.out.println("Rayon : " + R);
    }

   /**
    * methode print
    * enregistre les caractristiques de la sphre dans le flux spcifi
    * @param out flux de sortie
    */
    public void print(PrintWriter out) {
        for (double theta=0; theta<Math.PI; theta+=Math.PI/20) {
            for (double phi=0; phi<Math.PI*2; phi+=Math.PI/20) {
                out.print(R*Math.sin(theta)*Math.cos(phi)); out.print(" "); out.print(R*Math.sin(theta)*Math.sin(phi)); out.print(" "); out.print(R*Math.cos(theta)); out.println();
            }
        }
    }

   /**
    * methode distance_max
    * retourne le rayon de la sphre, centre sur le point donn, circonscrite  la forme gomtrique
    * @param P le point de rfrence
    * @return le rayon de la sphre, centre sur le point donn, circonscrite  la forme gomtrique
    */
    public double distance_max(Point P) {
        return R+P.distance(O);
    }

   /**
    * methode rotated_x
    * tourne la forme 3D d'un angle donn autour de l'axe x
    * @param angle l'angle de rotation
    */
    public void rotated_x(double angle) {O.rotated_x(angle);}

   /**
    * methode rotated_y
    * tourne la forme 3D d'un angle donn autour de l'axe y
    * @param angle l'angle de rotation
    */
    public void rotated_y(double angle) {O.rotated_y(angle);}

   /**
    * methode rotated_z
    * tourne la forme 3D d'un angle donn autour de l'axe z
    * @param angle l'angle de rotation
    */
    public void rotated_z(double angle) {O.rotated_z(angle);}

   /**
    * methode in_projection_z
    * retourne si le point P est inclu dans la forme gomtrique projete selon z
    * @return un booleen indiquant si le point P est inclu dans la forme gomtrique  projete selon z
    */
    public boolean in_projection_z(Point2D P) {
        if (Math.pow(P.get_x()-O.get_x(),2)+Math.pow(P.get_y()-O.get_y(),2)<=R*R) return true; else return false;
    }

   /**
    * methode draw
    * retourne l'objet graphique pass en argument aprs ajout de la reprsentation de la forme gomtrique
    * @param g       l'objet graphique
    * @param facteur le facteur de grandissement  appliquer
    * @return l'objet graphique aprs ajout de la reprsentation de la forme gomtrique
    */    
    public Graphics draw(Graphics g, double facteur) {
        g.drawOval((int)Math.floor((O.get_y()+R)*facteur), (int)Math.floor((O.get_z()+R)*facteur), (int)Math.floor(2*R*Math.abs(facteur)), (int)Math.floor(2*R*Math.abs(facteur)));
        return g;
    }
    
}