package geometrie;

/**
 * classe TrapezeIsocele
 * classe reprsentant un trapze, troncature d'un triangle isocle, isocle dans l'espace
 */
 
public class TrapezeIsocele extends Polygone {

    private double base, hauteur, hauteur_triangle; //base et hauteur du trapze isocle et hauteur du triangle servant de base au trapze
    
   /**
    * constructeur TrapezeIsocele 
    * construit un trapze isocle avec les paramtres spcifis
    * @param base_              la longueur du trapze
    * @param hauteur_           la hauteur du trapze
    * @param hauteur_triangle_  la hauteur du triangle servant de base au trapze
    * @param centre             le centre du trapze
    * @param n_                 la normale au trapze
    */
    public TrapezeIsocele(double base_, double hauteur_, double hauteur_triangle_, Point centre, Vecteur n_) {
        super(centre, n_);
        base=base_;
        hauteur=hauteur_;
        hauteur_triangle = hauteur_triangle_;
        calculeCoins();
    }
    
   /**
    * constructeur TrapezeIsocele 
    * construit un trapze isocle avec les paramtres spcifis
    * @param base_              la longueur du trapze
    * @param hauteur_           la hauteur du trapze
    * @param hauteur_triangle_  la hauteur du triangle servant de base au trapze
    * @param centre             le centre du trapze
    */
    public TrapezeIsocele(double base_, double hauteur_, double hauteur_triangle_, Point centre, Angles3 a) {
        super(centre, new Vecteur(0,0,1));
        base=base_;
        hauteur=hauteur_;
        hauteur_triangle = hauteur_triangle_;
        calculeCoins(a);
    }
    
   /**
    * methode calculeCoins
    * calcule la position de chaque coin en fonction des paramtres du trapze
    */
    protected void calculeCoins() {
        //On considre un trapze dans le plan z=0
        Vecteur OP = (new Point(0,0,0)).vecteur(P);
        double phi = n.get_phi();
        double theta = n.get_theta();
        coins = new Point[4];
        coins[0] = new Point(0, -base/2., 0);
        coins[1] = new Point(0, base/2., 0);
        coins[2] = new Point(hauteur, base*hauteur/hauteur_triangle/2., 0);
        coins[3] = new Point(hauteur, -base*hauteur/hauteur_triangle/2., 0);

        //On tourne puis on translate
        for (int i=0; i<4; i++) {
            coins[i].rotated_y(theta);
            coins[i].rotated_z(phi);
            coins[i].translated(OP);
        }
    }
    
   /**
    * methode calculeCoins
    * calcule la position de chaque coin en fonction des paramtres du trapze
    * @param a les angles d'euler pour l'orientation du trapze
    */
    protected void calculeCoins(Angles3 a) {
        //On considre un trapze dans le plan z=0
        Vecteur OP = (new Point(0,0,0)).vecteur(P);
        coins = new Point[4];
        coins[0] = new Point(0, -base/2., 0);
        coins[1] = new Point(0, base/2., 0);
        coins[2] = new Point(hauteur, base*hauteur/hauteur_triangle/2., 0);
        coins[3] = new Point(hauteur, -base*hauteur/hauteur_triangle/2., 0);

        n.rotated_z(a.get_psi());
        n.rotated_y(a.get_theta());
        n.rotated_z(a.get_phi());
        
        //On tourne puis on translate
        for (int i=0; i<4; i++) {
            coins[i].rotated_z(a.get_psi()); 
            coins[i].rotated_y(a.get_theta()); 
            coins[i].rotated_z(a.get_phi()); 
            coins[i].translated(OP); 
        }
    }

}