package geometrie;

/**
 * classe TriangleIsocele
 * classe reprsentant un triangle isocle dans l'espace
 */
 
public class TriangleIsocele extends Polygone {

    private double base, hauteur; //base et hauteur du triangle isocle
    
   /**
    * constructeur TriangleIsocele 
    * construit un triangle isocle avec les paramtres spcifis
    * @param base_      la longueur du triangle
    * @param hauteur_   la largeur du triangle
    * @param centre     le centre du triangle
    * @param n_         la normale au triangle
    */
    public TriangleIsocele(double base_, double hauteur_, Point centre, Vecteur n_) {
        super(centre, n_);
        base=base_;
        hauteur=hauteur_;
        calculeCoins();
    }
    
   /**
    * constructeur TriangleIsocele 
    * construit un triangle isocle avec les paramtres spcifis
    * @param base_      la longueur du triangle
    * @param hauteur_   la largeur du triangle
    * @param centre     le centre du triangle
    */
    public TriangleIsocele(double base_, double hauteur_, Point centre, Angles3 a) {
        super(centre, new Vecteur(0,0,1));
        base=base_;
        hauteur=hauteur_;
        calculeCoins(a);
    }
    
   /**
    * methode calculeCoins
    * calcule la position de chaque coin en fonction des paramtres du triangle
    */
    protected void calculeCoins() {
        //On considre un triangle dans le plan z=0
        Vecteur OP = (new Point(0,0,0)).vecteur(P);
        double phi = n.get_phi();
        double theta = n.get_theta();
        coins = new Point[3];
        coins[0] = new Point(0, -base/2., 0);
        coins[1] = new Point(0, base/2., 0);
        coins[2] = new Point(hauteur, 0, 0);

        //On tourne puis on translate
        for (int i=0; i<3; i++) {
            coins[i].rotated_y(theta);
            coins[i].rotated_z(phi);
            coins[i].translated(OP);
        }
    }
    
   /**
    * methode calculeCoins
    * calcule la position de chaque coin en fonction des paramtres du triangle
    * @param a les angles d'euler pour l'orientation du triangle
    */
    protected void calculeCoins(Angles3 a) {
        //On considre un triangle dans le plan z=0
        Vecteur OP = (new Point(0,0,0)).vecteur(P);
        coins = new Point[3];
        coins[0] = new Point(0, -base/2., 0);
        coins[1] = new Point(0, base/2., 0);
        coins[2] = new Point(hauteur, 0, 0);

        n.rotated_z(a.get_psi());
        n.rotated_y(a.get_theta());
        n.rotated_z(a.get_phi());
        
        //On tourne puis on translate
        for (int i=0; i<3; i++) {
            coins[i].rotated_z(a.get_psi()); 
            coins[i].rotated_y(a.get_theta()); 
            coins[i].rotated_z(a.get_phi()); 
            coins[i].translated(OP); 
        }
    }

}