package halos;

import java.math.*;
import geometrie.*;
import java.awt.Graphics;

/**
 * classe Cristal
 * classe modlisant un Cristal de glace
 */
 
public class Cristal {
    

    protected double theta=0, phi=0, psi=0; //orientation du cristal dans l'espace
    protected Forme3D[] faces;              //tableau contenant les faces du cristal
    protected double rayon;                 //rayon de la sphre circonscrite au cristal
    protected double n_cristal;             //indice optique du cristal
    
    protected double n_air=1;               //indice optique de l'air
    protected int nombre_max_reflexions = 100;

   /**
    * constructeur Cristal
    * construit un objet cristal sans initialisation
    */
    public Cristal() {
        
    }
    
   /**
    * constructeur Cristal
    * construit un cristal de type : (1) hexagonal
    * @param type           le type du cristal (1=hexagonal, 4=sphrique, 5=bipyramidal)
    * @param param_forme    le paramtre de forme
    * @param indice_optique l'indice optique du cristal
    */
    public Cristal(int type, double param_forme, double indice_optique) {

        n_cristal = indice_optique;
    
        if (type==1) { //hexagonal
            double a=1., h=a*param_forme;
            faces = new Forme3D[8];
            for (int i=0; i<6; i++) {
                faces[i] = new Rectangle(h, a, new Point(Math.sqrt(3)*a/2*Math.cos(Math.PI/6+i*Math.PI/3), Math.sqrt(3)*a/2*Math.sin(Math.PI/6+i*Math.PI/3), 0), new Vecteur(Math.cos(Math.PI/6+i*Math.PI/3), Math.sin(Math.PI/6+i*Math.PI/3), 0));
                
            }
            faces[6] = new Hexagone(a, new Point(0, 0, h/2.), new Vecteur(0,0,1));
            faces[7] = new Hexagone(a, new Point(0, 0, -h/2.), new Vecteur(0,0,-1));
        }

        if (type==2) { //pyramidal
            double angle = 28 * Math.PI / 180;
            double a=1., h=a*param_forme;
            faces = new Forme3D[18];
            for (int i=0; i<6; i++) {
                faces[i] = new Rectangle(h, a, new Point(Math.sqrt(3)*a/2*Math.cos(Math.PI/6+i*Math.PI/3), Math.sqrt(3)*a/2*Math.sin(Math.PI/6+i*Math.PI/3), 0), new Vecteur(Math.cos(Math.PI/6+i*Math.PI/3), Math.sin(Math.PI/6+i*Math.PI/3), 0));
                
            }
            
            //les 12 triangles : 
            double H = Math.sqrt(3)/2*a/Math.sin(angle); int j;
            for (int i=0; i<12; i++) {
                j=(int)Math.floor(Math.abs(i-6));
                faces[i+6] = new TriangleIsocele(a, H, new Point(Math.sqrt(3)*a/2*Math.cos(Math.PI/6+j*Math.PI/3), Math.sqrt(3)*a/2*Math.sin(Math.PI/6+j*Math.PI/3), -h/2 * signe(i-5.5)), new Angles3((Math.PI/2 + angle) * signe(i-5.5), Math.PI/6+j*Math.PI/3, 0));
            }
        }

        if (type==3) { //pyramidal tronqu
            double angle = 28 * Math.PI / 180;
            double a=1., h=a*param_forme;
            faces = new Forme3D[20];
            for (int i=0; i<6; i++) {
                faces[i] = new Rectangle(h, a, new Point(Math.sqrt(3)*a/2*Math.cos(Math.PI/6+i*Math.PI/3), Math.sqrt(3)*a/2*Math.sin(Math.PI/6+i*Math.PI/3), 0), new Vecteur(Math.cos(Math.PI/6+i*Math.PI/3), Math.sin(Math.PI/6+i*Math.PI/3), 0));
                
            }
            
            //les 12 trapezes : 
            double H = Math.sqrt(3)/2*a/Math.sin(angle); int j;
            for (int i=0; i<12; i++) {
                j=(int)Math.abs(Math.floor(i-6));
                faces[i+6] = new TrapezeIsocele(a, H/2, H, new Point(Math.sqrt(3)*a/2*Math.cos(Math.PI/6+j*Math.PI/3), Math.sqrt(3)*a/2*Math.sin(Math.PI/6+j*Math.PI/3), -h/2 * signe(i-5.5)), new Angles3((Math.PI/2 + angle) * signe(i-5.5), Math.PI/6+j*Math.PI/3, 0));
            }
            
            //les 2 hexagones : 
            faces[18] = new Hexagone(a/2, new Point(0, 0, h/2.+H/2*Math.cos(angle)), new Vecteur(0,0,1));                
            faces[19] = new Hexagone(a/2, new Point(0, 0, -h/2.-H/2*Math.cos(angle)), new Vecteur(0,0,1));              
        }

        if (type==4) { //sphre (goutte d'eau)
            faces = new Forme3D[1];
            faces[0] = new Sphere(new Point(0, 0, 0), 1);
        }

        if (type==5) { //bipyramidal
            double h=1., b=h/Math.sqrt(param_forme*param_forme/2+1./4), alpha=Math.acos(1./Math.sqrt(param_forme*param_forme*2+1));
            faces = new Forme3D[8];
            for (int i=0; i<8; i++) {
                faces[i] = new TriangleIsocele(b, h, new Point(((i+1)%2)*b/2*signe((i+1)%4-2),(i%2)*b/2*signe(i%4-2),0), new Angles3(alpha * signe(i-3.5), i*Math.PI/2, 0));
                
            }
        }
        
        calcule_rayon();
    }

   /**
    * methode get_rayon
    * retourne le rayon de la sphre circonscrite au cristal
    * @return le rayon de la sphre circonscrite au cristal
    */
    public double get_rayon() {return rayon;}

   /**
    * methode get_face
    * retourne la face demande du cristal
    * @param i le numro de face
    * @return la i-me face
    */
    public Forme3D get_face(int i) { return faces[i]; }

    /**
    * methode get_nombre_faces
    * retourne le nombre de faces du cristal
    * @return le nombre de faces du cristal
    */
    public int get_nombre_faces() {return faces.length;}

    /**
    * methode set_indice
    * permet de modifier l'indice optique du cristal
    * @param n l'indice optique du cristal
    */
    public void set_indice(double n) {n_cristal=n;}

   /**
    * methode rotated_x
    * tourne le cristal autour de l'axe x
    * @param angle l'angle de rotation
    */
    protected void rotated_x(double angle) {
        for (int i=0; i<faces.length; i++) (faces[i]).rotated_x(angle);
    }

   /**
    * methode rotated_y
    * tourne le cristal autour de l'axe y
    * @param angle l'angle de rotation
    */
    protected void rotated_y(double angle) {
        for (int i=0; i<faces.length; i++) (faces[i]).rotated_y(angle);
    }

   /**
    * methode rotated_z
    * tourne le cristal autour de l'axe z
    * @param angle l'angle de rotation
    */
    protected void rotated_z(double angle) {
        for (int i=0; i<faces.length; i++) (faces[i]).rotated_z(angle);
    }

   /**
    * methode oriente
    * tourne le cristal selon les angles theta, phi, psi indiqu
    * @param a l'objet Angles3 contenant les 3 angles
    */
    public void oriente(Angles3 a) {
        rotated_z(-phi);            //on revient en position neutre
        rotated_y(-theta);
        rotated_z(a.get_psi()-psi); //puis on tourne
        rotated_y(a.get_theta());
        rotated_z(a.get_phi());
        theta = a.get_theta();      //et on sauvegarde les valeurs
        phi = a.get_phi();
        psi = a.get_psi();
    }

   /**
    * methode tourne
    * tourne le cristal des angles theta, phi, psi indiqu par rapport  sa position prcdente
    * @param dtheta l'angle theta
    * @param dphi   l'angle phi
    * @param dpsi   l'angle psi
    */
    public void tourne(double dtheta, double dphi, double dpsi) {
        rotated_z(-phi);            //on revient en position neutre
        rotated_y(-theta);
        rotated_z(dpsi);            //puis on tourne
        rotated_y(theta+dtheta);
        rotated_z(phi+dphi);
        theta += dtheta;      //et on sauvegarde les nouvelles valeurs
        phi += dphi;
        psi += dpsi;
    }

   /**
    * methode set_reference
    * rinitialise les angles  0
    */
    public void set_reference() {
        theta = 0; phi = 0; psi = 0;
    }

   /**
    * methode calcule_rayon
    * calcule le rayon de la sphre circonscrite au cristal
    */
    private void calcule_rayon() {
        double d;
        rayon = 0;
        Point O = new Point(0,0,0);
        for(int i=0; i<faces.length; i++) {
            d=(faces[i]).distance_max(O);
            if (d>rayon) rayon=d;
        }
    }

   /**
    * methode facesuivante 
    * retourne l'indice de la face suivante qui interceptera le rayon lumineux
    * @param r                  le rayon lumineux
    * @param face_precedente    l'indice de la face prcdente
    * @return l'indice de la face suivante qui interceptera le rayon lumineux
    */
    private int facesuivante(Rayon r, int face_precedente) {
        int face_suivante = -1;
        double d, distance_min=0;
        for (int i=0; i<get_nombre_faces(); i++) {
        /*** attention, ne marchera pas dans le cas d'une sphre seule : il n'y aura qu'une seule face et 2 intersections possibles ***/ 
            //if (i!=face_precedente) {
                d=get_face(i).distance_intersect(r.get_d());
                if ( (face_suivante==-1 || d<distance_min) && d>1e-5) {face_suivante = i; distance_min = d;}
            //}
        }
        return face_suivante;
    }

   /**
    * methode vecteur_sortie 
    * retourne le vecteur de sortie du rayon lumineux aprs interaction avec le cristal
    * @param r                  le rayon lumineux
    * @param angle              l'angle d'incidence du rayon lumineux par rapport au cristal
    * @param nb_reflexions      le nombre de reflexions internes imposes
    * @return le vecteur de sortie du rayon lumineux aprs interaction avec le cristal
    */
    public Vecteur vecteur_sortie(Rayon r, double angle, int nb_reflexions) {

            rotated_y(angle);   //rotation du cristal

            int face_precedente = -1, face_suivante;
            boolean refraction;
            double d, distance_min=0;
            int n_boucle = 0;
    
            //Recherche de la face intersecte en premier
            Point2D p2d = new Point2D(r.get_P());
            Forme3D f;
            
            face_suivante = -1;
            for (int i=0; i<get_nombre_faces(); i++) {
                f = get_face(i);
                if(f.in_projection_z(p2d)) {
                    d = f.distance_intersect(r.get_d());
                    if (face_suivante == -1 || (d<distance_min) ) {distance_min = d; face_suivante = i;}
                }
            }

            if (face_suivante==-1) {rotated_y(-angle); return new Vecteur(0,0,0);} //si le rayon n'atteint pas le cristal

            r=r.refract(get_face(face_suivante), 1, n_air, n_cristal); //rfraction sur la face d'entre

            //bouclage sur les rfractions et rflexions suivantes
            do {
                //Recherche de la face intersecte ensuite
                n_boucle++;
                if (n_boucle>nombre_max_reflexions) {rotated_y(-angle); return new Vecteur(0,0,0);} //le rayon subit des rflexions internes en boucle
                
                face_precedente = face_suivante;
                face_suivante = facesuivante(r, face_precedente);

                if (face_suivante==-1) {rotated_y(-angle); return new Vecteur(0,0,0);} //en cas de problme lors du calcul

                if ( (refraction = r.is_refract(get_face(face_suivante), n_cristal, n_air)) && (nb_reflexions<1))  //si rfraction
                    r=r.refract(get_face(face_suivante), 1, n_cristal, n_air); //rfraction sur la face suivante
                else {
                    r=r.reflect(get_face(face_suivante), 1); //sinon rflexion
                    if (refraction) nb_reflexions--;
                    refraction = false;
                }
            } while (!refraction);
            
        rotated_y(-angle);  //rorientation du cristal
        return r.get_v();
    }

   /**
    * methode trajectoire 
    * retourne une liste de points reprsentant la trajctoire du rayon lumineux dans le cristal
    * @param r                  le rayon lumineux
    * @param angle              l'angle d'incidence du rayon lumineux par rapport au cristal
    * @param nb_reflexions      le nombre de reflexions internes imposes
    * @return une liste de points reprsenatnt la trajctoire du rayon lumineux dans le cristal
    */
    public Point[] trajectoire(Rayon r, double angle, int nb_reflexions) {

            rotated_y(angle);   //rotation du cristal

            Point[] liste = new Point[nombre_max_reflexions+5];
            int n=0;
            int n_boucle=0;

            int face_precedente = -1, face_suivante;
            boolean refraction;
            double d, distance_min=0;
    
            //Recherche de la face intersecte en premier
            Point2D p2d = new Point2D(r.get_P());
            Forme3D f;
            
            liste[n++] = r.get_P();
            
            face_suivante = -1;
            for (int i=0; i<get_nombre_faces(); i++) {
                f = get_face(i);
                if(f.in_projection_z(p2d)) {
                    d = f.distance_intersect(r.get_d());
                    if (face_suivante == -1 || (d<distance_min) ) {distance_min = d; face_suivante = i;}
                }
            }

            if (face_suivante==-1) {rotated_y(-angle); return new Point[0];} //si le rayon n'atteint pas le cristal

            liste[n++] = get_face(face_suivante).intersect(r.get_d());
            
            r=r.refract(get_face(face_suivante), 1, n_air, n_cristal); //rfraction sur la face d'entre

            //bouclage sur les rfractions et rflexions suivantes
            do {
                n_boucle++;
                if (n_boucle>nombre_max_reflexions) {rotated_y(-angle); return new Point[0];} //le rayon subit des rflexions internes en boucle
                
                //Recherche de la face intersecte ensuite
                face_precedente = face_suivante;
                face_suivante = facesuivante(r, face_precedente);

                if (face_suivante==-1) {rotated_y(-angle); return new Point[0];} //en cas de problme lors du calcul

                liste[n++] = get_face(face_suivante).intersect(r.get_d());
                
                if ( (refraction = r.is_refract(get_face(face_suivante), n_cristal, n_air)) && (nb_reflexions<1))  //si rfraction
                    r=r.refract(get_face(face_suivante), 1, n_cristal, n_air); //rfraction sur la face suivante
                else {
                    r=r.reflect(get_face(face_suivante), 1); //sinon rflexion
                    if (refraction) nb_reflexions--;
                    refraction = false;
                }
            } while (!refraction);

            liste[n++] = get_face(face_suivante).intersect(r.get_d());
            liste[n] = liste[n-1].translate(r.get_v().fois(2*rayon)); n++;
            
            Point[] listef = new Point[n];
            for (int i=0; i<n; i++) listef[i]=liste[i];
            
        rotated_y(-angle);  //rorientation du cristal
        return listef;
    }

    public void affiche() {
        System.out.println("Cristal :");
        System.out.println("angles : "+theta+" "+phi+" "+psi);
    }
    
   /**
    * methode signe
    * retourne le signe de la variable
    * @param x la variable a tester
    * @return le signe de la variable
    */
    public double signe(double x) {
        if (x>0) return 1;
        if (x<0) return -1;
        return 0;
    }
    
   /**
    * methode draw
    * retourne l'objet graphique pass en argument aprs ajout de la reprsentation du cristal
    * @param g       l'objet graphique
    * @param facteur le facteur de grandissement  appliquer
    * @return l'objet graphique aprs ajout de la reprsentation du cristal
    */    
    public Graphics draw(Graphics g, double facteur) {
        for (int i=0; i<faces.length; i++) faces[i].draw(g, facteur);
        return g;
    }
    
}
