package halos;

import simu.*;
import java.math.*;
import java.util.Vector;
import geometrie.*;

import halos.*;

import java.awt.image.*;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Color;
import java.lang.String;

/**
 * classe Halos
 * applet calculant des halos atmosphriques par simulation des interactions de cristaux de glace avec la lumire du soleil
 */
 
public class Halos extends SimuApplet {
    
   /**
    * methode create_image 
    * cr une image  partir de 3 tableaux contenant les valeurs RVB.
    * Un filtre logarithmique est appliqu et la renormalisation est faite sur les pixels pour lesquels le masque vaut 0
    * @param rrr    le tableau de la couche rouge de l'image
    * @param vvv    le tableau de la couche verte de l'image
    * @param bbb    le tableau de la couche bleue de l'image
    * @param vmax   la valeur maximale  donner pour la renormalisation (comprise entre 0 et 255)
    * @param masque le tableau de masque, pour les pixels  prendre en compte pour la renormalisation
    * @return       l'image cre
    */
     Image create_image(double rrr[][],double vvv[][],double bbb[][], double vmax, int[][] masque) {
        
        double max=0;
        int RR, VV, BB, c;
        int taille = rrr.length;
        Image img = createImage(taille, taille);
        Graphics g = (img).getGraphics();
        
        double rr[][] = new double[taille][taille];
        double vv[][] = new double[taille][taille];
        double bb[][] = new double[taille][taille];
        
        //filtre logarithmique
        for (int i=0; i<taille; i++) {              
            for (int j=0; j<taille; j++) {
                rr[i][j] = Math.log(rrr[i][j]+1);
                vv[i][j] = Math.log(vvv[i][j]+1);
                bb[i][j] = Math.log(bbb[i][j]+1);
            }
        }        
        
        //renormalisation
        for (int i=0; i<taille; i++) {              
            for (int j=0; j<taille; j++) {
                if (masque[i][j] !=0 ) continue;    //points qui seront masqus d'une faon ou une autre
                if (rr[i][j]>max) max=rr[i][j];
                if (vv[i][j]>max) max=vv[i][j];
                if (bb[i][j]>max) max=bb[i][j];
            }
        }
        
        if (max==0) max=1;
        
        //affichage
        for (int i=0; i<taille; i++) {
            for (int j=0; j<taille; j++) {
                if (masque[i][j] !=0 ) continue;    //points qui seront masqus d'une faon ou une autre
                RR = (int)Math.floor(rr[i][j]/max*vmax);
                VV = (int)Math.floor(vv[i][j]/max*vmax);
                BB = (int)Math.floor(bb[i][j]/max*vmax);
                if (RR<0) System.out.println(rr[i][j]);
                g.setColor(new Color(RR,VV,BB));
                g.drawLine(taille-i-1,taille-j-1,taille-i-1,taille-j-1);
            }
        }
        
        return img;
    }
    
   /**
    * methode addPoints 
    * ajoute  une image des points de couleur donne
    * @param img        l'image  modifier
    * @param point      le tableau de mme taille que l'image contenant le code couleur (indice des couleurs de la liste "couleur") des points  ajouter
    * @param couleur    la liste des couleurs
    * @param taille     la taille de l'image, et du tableau
    */
     void addPoints(Image img, int[][] point, Color[] couleur, int taille) {
        Graphics g = (img).getGraphics();
        for(int i=0; i<taille; i++) {
            for(int j=0; j<taille; j++) {
                if (point[i][j]!=0) {
                    g.setColor(couleur[point[i][j]-1]);
                    g.drawLine(taille-i-1,taille-j-1,taille-i-1,taille-j-1);
                }
            }
        }
    }
    

     double hauteur, zoom;                                               //hauteur du soleil et zoom pour l'affichage
     int taille_image = 512;                                             //taille de la zone d'affichage en pixels
     int [] histo = new int[181];                                        //histogramme des valeurs des angles calculs
    
     double Rt;                                                          //rayon terrestre
     double altitude_nuages;                                             //altitude des nuages de glace
     double atm;                                                         //distance des nuages au centre de la Terre en rayons terrestres
    
     Point I = new Point();                                              //point d'intersection des rayons dans les nuages
     Point O;                                                            //coordonnes du centre de la Terre
     Point V = new Point(0, 0, 0);                                       //coordonnes de l'observateur
     Vecteur u, v = new Vecteur();                                       //vecteurs des rayons solaire et ligne de vise de l'observateur

     int type_cristal;                                                   //type de cristal etudi
     double facteur_forme;                                               //facteur de forme du cristal
     int nb_reflexions;                                                  //nombre de rflexions internes supplmentaires imposes
     int type_histo;                                                     //type d'histogramme : 0 rien, 1 centr sur le soleil, 2 centr sur le zenith

     double azimut_centre, hauteur_centre;                               //centre de la projection

     double facteur_zoom;

     double [][][] tableau = new double[3][taille_image][taille_image];  //tableau contenant l'image calcule
     boolean reinitialiser;                                              //indique si l'image et l'histogramme doivent tre rinitialiss

     Vecteur v0 = new Vecteur(0,0,-1);                                   //vecteur du rayon de rfrence pour les calculs de rfraction
     Angles3 angles_c = new Angles3();                                   //angles d'orientation du cristal

     Point2D P = new Point2D();;                                         //Point pour les calculs de projection

     Cristal c;                                                          //Cristal tudi
     Rayon r;                                                            //Rayon test

     double R, R2;                                                       //la rayon et diamtre du cristal tudi

     int type_aleatoire;                                                 //type de fonction alatoire pour le cristal : 0 alatoire, 1 "plate", 2 "column", 3 "column2"

     double[] indices = new double[3];                                   //indices du milieu, pour quelques longueurs d'onde donnes

     double azimut_centre_cercles, hauteur_centre_cercles, pas_cercles;  //donnes pour les cercles
     int[][] points_cercle = new int[taille_image][taille_image];        //liste de points pour affichage des cecles (1) et sous l'horizon (2)
     Color[] couleurs_cercle = {new Color(255,255,255), new Color(0,0,20), new Color(255, 255, 0)};     //couleurs associes

     boolean tracer_cercles, cacher_horizon;                             //choix de l'utilisateur pour l'affichage des cercles et de l'image sous l'horizon
     int nb_rayons;                                                      //nombre de rayons par itration
     boolean afficher_soleil;                                            //choix de l'affichage du soleil
    
     double cos_imax;                                                    //cos de l'angle de la Terre vu d'un cristal



   /**
    * constructeur Halos 
    */
    public Halos() {
        appletDoc("halos_doc.xml");
    }
    
   /**
    * methode verifier 
    * verifie les paramtres entrs par l'utilisateur et provoque une exception en cas de problme
    */
    public void verifier() throws SimuException {
        boolean erreur = false;
        String message = "";
        if (hauteur>90 || hauteur<-90) {erreur = true; message += " hauteur en dehors des valeurs autorises,";}
        if (zoom>180 || zoom<=0) {erreur = true; message += " zoom en dehors des valeurs autorises,";}
        if (hauteur_centre>90 || hauteur_centre<-90) {erreur = true; message += " hauteur du centre en dehors des valeurs autorises,";}
        if (tracer_cercles && (hauteur_centre>90 || hauteur_centre<-90)) {erreur = true; message += " hauteur du centre des cercles en dehors des valeurs autorises,";}

        if (erreur) throw new SimuException(message);
    }
    
   /**
    * methode initCalculLive 
    * initialise l'applet
    * @param in liste des valeurs entres par l'utilisateur
    */
    public void initCalculLive(ListeValeurs in) throws SimuException {

        //lecture des paramtres d'entre
        String str;

        Rt = 6.4e6;;
        altitude_nuages = 5000;

        nb_rayons = (int)in.lireDouble("rayons");
        hauteur=in.lireDouble("hauteur");
        zoom = in.lireDouble("zoom");

        str = in.lireString("typecristal");
        if ("cristal hexagonal".equals(str)) type_cristal = 1;
        if ("cristal pyramidal".equals(str)) type_cristal = 2;
        if ("cristal pyramidal tronqu".equals(str)) type_cristal = 3;
        if ("goutte sphrique".equals(str)) type_cristal = 4;
        if ("cristal bipyramidal".equals(str)) type_cristal = 5;

        facteur_forme = in.lireDouble("facteurforme");

        str = in.lireString("typealeatoire");
        if ("Alatoire".equals(str)) type_aleatoire = 0;
        if ("Alatoire, axe principal vertical".equals(str)) type_aleatoire = 1;
        if ("Alatoire, axe principal horizontal".equals(str)) type_aleatoire = 2;
        if ("Alatoire, orientation de Parry".equals(str)) type_aleatoire = 3;

        nb_reflexions = (int)in.lireDouble("reflexions");
        str = in.lireString("materiau");
        if ("glace".equals(str)) {
            indices[0] = 1.308;
            indices[1] = 1.313;
            indices[2] = 1.316;
        }
        if ("eau".equals(str)) {
            indices[0] = 1.3318;
            indices[1] = 1.3360;
            indices[2] = 1.3417;
        }

        str = in.lireString("typehisto");
        if ("aucun".equals(str)) type_histo = 0;
        if ("centr sur le soleil".equals(str)) type_histo = 1;
        if ("centr sur le znith".equals(str)) type_histo = 2;

        azimut_centre = in.lireDouble("azimutcentre");
        hauteur_centre = in.lireDouble("hauteurcentre");
        str = in.lireString("cercles");
        if ("on".equals(str)) tracer_cercles = true; else tracer_cercles = false;
        pas_cercles = in.lireDouble("pascercles");
        azimut_centre_cercles = in.lireDouble("azimutcercles");
        hauteur_centre_cercles = in.lireDouble("hauteurcercles");
        str = in.lireString("soleil");
        if ("on".equals(str)) afficher_soleil = true; else afficher_soleil = false;
        str = in.lireString("horizon");
        if ("on".equals(str)) cacher_horizon = true; else cacher_horizon = false;
        str = in.lireString("reinitialiser");
        if ("on".equals(str)) reinitialiser = true; else reinitialiser = false;

        verifier();     //vrifie la validit des paramtres entrs
        
        //suppression des donnes des tableaux
        if (reinitialiser) {
        
            for (int i=0; i<taille_image; i++) {
                for(int j=0; j<taille_image; j++) {
                    for(int k=0; k<3; k++) {
                        tableau[k][i][j] = 0;
                    }
                    points_cercle[i][j] = 0;
                }
            }
            
            for (int i=0; i<histo.length; i++) histo[i] = 0;
        
        }
        
        //conversions degres-> radians et calcul des paramtres utiles au programme
        hauteur*=Math.PI/180;
        zoom*=Math.PI/180;
        azimut_centre*=Math.PI/180;
        hauteur_centre*=Math.PI/180;
        azimut_centre_cercles*=Math.PI/180;
        hauteur_centre_cercles*=Math.PI/180;
        pas_cercles*=Math.PI/180;
        u = new Vecteur(-Math.cos(hauteur), 0, -Math.sin(hauteur));
        facteur_zoom = taille_image/(2*Math.sin(zoom/2));
        c = new Cristal(type_cristal, facteur_forme, 1.3);
        R = c.get_rayon(); R2=R*2;
        r = new Rayon(v0, new Point(0,0,R2), 1);
        atm = (Rt+altitude_nuages)/Rt;
        cos_imax = Math.sqrt(1-1/atm/atm);
        O = new Point(0, 0, -Rt);
        

        //cration du tableau contenant l'horizon, le soleil et les cercles
        for(int i=0; i<taille_image; i++) {
            for(int j=0; j<taille_image; j++) {
                points_cercle[i][j]=0;
            }
        }

        if (tracer_cercles) {
            int X,Y;
            Vecteur tmp = new Vecteur();
            for (double i=pas_cercles; i<=Math.PI; i+=pas_cercles) {
                for (double j=0; j<6.4; j+=zoom/taille_image/2) {
                    tmp.set_euler(i, j);
                    tmp.rotated_y(Math.PI/2-hauteur_centre_cercles);
                    tmp.rotated_z(azimut_centre_cercles);
                    P = projection(azimut_centre, hauteur_centre, tmp.get_phi(), Math.PI/2-tmp.get_theta(), facteur_zoom);
                    if (P == null) continue;
                    X=(int)Math.floor(P.get_x()+taille_image/2);
                    Y=(int)Math.floor(P.get_y()+taille_image/2);
                    if (X>=0 && Y>=0 && X<taille_image && Y<taille_image) points_cercle[X][Y] = 1;
                }
            }
        }

        if (afficher_soleil) {
            int X,Y;
            double i = 0.25 * Math.PI / 180;
            Vecteur tmp = new Vecteur();
                for (double j=0; j<6.4; j+=zoom/taille_image/2) {
                    tmp.set_euler(i, j);
                    tmp.rotated_y(Math.PI/2-hauteur);
                    P = projection(azimut_centre, hauteur_centre, tmp.get_phi(), Math.PI/2-tmp.get_theta(), facteur_zoom);
                    if (P == null) continue;
                    X=(int)Math.floor(P.get_x()+taille_image/2);
                    Y=(int)Math.floor(P.get_y()+taille_image/2);
                    //System.out.println(X+" "+Y);
                    //break;
                    if (X>=0 && Y>=0 && X<taille_image && Y<taille_image) {points_cercle[X][Y] = 3;};
                }
        }        
        
        if (cacher_horizon) {
            for(int i=0; i<taille_image; i++) {
                for(int j=0; j<taille_image; j++) {
                    if (Math.PI/2-de_projection_orthographique(azimut_centre, hauteur_centre, i, j, taille_image, zoom).get_theta()<0) {points_cercle[i][j]=2;};
                }
            }
        }

        
    }
    
   /**
    * methode calculLive 
    * effectue le calcul d'un certain nombre de rayons lumineux avant de retourner le rsultat
    * @return l'image et l'histogramme du halo
    */
    public ListeValeurs calculLive() throws SimuException {
    
        ListeValeurs out = new ListeValeurs();
    
        double cos_gamma, cos_theta, theta;  //quelques angles utiles ...
        int X, Y; //indice du point sur l'image

        int k; //couleur
        double x=0,y=0; //coordonnes du rayon
        
        
        for (int i=0;i<nb_rayons;i++) {

            do {
                if (type_aleatoire==0) angles_c.random();
                if (type_aleatoire==1) angles_c.random_plate(0.4);
                if (type_aleatoire==2) angles_c.random_column(0.4);
                if (type_aleatoire==3) angles_c.random_column2(0.4);
                c.oriente(angles_c);

                k=(int)Math.floor((Math.random())*3);           //choix alatoire des couleurs
                c.set_indice(indices[k]);

                do {                                            // tirage alatoire d'une position sur le cristal
                    x=(Math.random()-0.5)*R2;
                    y=(Math.random()-0.5)*R2;
                } while (x*x+y*y>R*R);                          // retirage si le rayon ne tombe pas dans le cercle de rayon le cristal

                r.set_P(x, y, R2);
                v = c.vecteur_sortie(r, hauteur-Math.PI/2, nb_reflexions);  //calcul de l'intraction

            } while (v.norme()<0.1); //ie norme(v) = 0, le rayon n'a pas atteint le cristal ou il y a eu un problme lors du calcul

            if (v.get_theta()>0.999*Math.PI) continue;  //rayon non dvi, on passe au suivant

            if (type_histo==1) histo[(int)(Math.acos(v.scalaire(v0))*180/Math.PI)] ++; // si histogramme centr autour du soleil

            v.rotated_y(Math.PI/2-hauteur);     //on roriente le vecteur de sortie (calcul pour un rayon incident vertical)
            
            if (type_histo==2) histo[(int)(Math.acos(v.scalaire(v0))*180/Math.PI)] ++; // si histogramme centr autour du zenith
            
            v.oppose();             //la ligne de vise est oppose au rayon mergent
            
            theta = v.get_theta();
            cos_theta = Math.cos(theta);
            I = V.translate(v.fois(Rt*(Math.sqrt(cos_theta*cos_theta+atm*atm-1)-cos_theta)));   //calcul de la position du cristal
            
            cos_gamma = u.scalaire(O.vecteur(I)) / (Rt * atm);
            if (cos_gamma>cos_imax) continue;                        //rayon incident solaire cach par la Terre

            // projette v et ajoute cos_gamma  l'histogramme si le rayon est dans l'cran
            P = projection(azimut_centre, hauteur_centre, v.get_phi(), Math.PI/2-v.get_theta(), facteur_zoom);
            if (P == null) continue;
            X=(int)Math.floor(P.get_x()+taille_image/2);
            Y=(int)Math.floor(P.get_y()+taille_image/2);
            if (X>=0 && Y>=0 && X<taille_image && Y<taille_image) tableau[k][X][Y] += r.get_I()*Math.abs(cos_gamma);
       }

        //cration de l'image  partir des tableaux de donne
        Image img = create_image(tableau[0], tableau[1], tableau[2], 255, points_cercle);
        addPoints(img, points_cercle, couleurs_cercle, taille_image);

        //envoi des donnes  simulab ...
        for (int i=0; i<histo.length; i++) {
            out.ajouterDouble("angle", i);
            out.ajouterDouble("hits", histo[i]);
        }

        //... avec un choix dpendant de la taille de l'image de sortie
        if ("affichage".equals(getAffichageChoisi().label)) out.ajouter("image", img);
        if ("affichage-histogramme".equals(getAffichageChoisi().label)) out.ajouter("image", img.getScaledInstance(256,256,Image.SCALE_DEFAULT));

        return out;
    }
    

   /**
    * methode de_projection_orthographique 
    * retourne les angles correspondant au point donn d'une projection orthographique
    * @param azimut_centre      l'azimut situ au centre de la projection
    * @param hauteur_centre     la hauteur situ au centre de la projection
    * @param X                  la coordonne en x
    * @param Y                  la coordonne en y
    * @param taille             la taille de la projection
    * @param Zoom               le facteur de zoom utilis
    * @return les angles correspondant au point donn sur la projection
    */
     Angles de_projection_orthographique(double azimut_centre, double hauteur_centre, double X, double Y, double taille, double Zoom) {
    
        double K = 2*Math.sin(Zoom/2)/taille;
    
        double x = (X-taille/2)*K;
        double y = (Y-taille/2)*K;
        
        double rho = Math.sqrt(x*x+y*y);
        double c = Math.asin(rho);
        double ha = Math.asin(Math.cos(c)*Math.sin(hauteur_centre)+y*Math.cos(hauteur_centre));
        double az = azimut_centre + Math.atan(x/(Math.cos(hauteur_centre)*Math.cos(c) - y*Math.sin(hauteur_centre)));
    
    //System.out.println(x+" "+y+" "+rho+" "+(taille/2)+" "+c);
    
        return new Angles(Math.PI/2-ha, az);
    }
    
   /**
    * methode projection 
    * retourne la projection orthographique des coordonnes donnes
    * @param azimut_centre      l'azimut situ au centre de la projection
    * @param hauteur_centre     la hauteur situ au centre de la projection
    * @param azimut             l'azimut  projeter
    * @param hauteur            la hauteur  projeter
    * @param facteur            le facteur de conversion (li au zoom souhait)
    * @return la projection orthographique des coordonnes donnes
    */
     public Point2D projection(double azimut_centre, double hauteur_centre, double azimut, double hauteur, double facteur) {
        if (Math.sin(hauteur_centre)*Math.sin(hauteur)+Math.cos(hauteur_centre)*Math.cos(hauteur)*Math.cos(azimut-azimut_centre)<0) return null;
        double x=facteur*Math.cos(hauteur)*Math.sin(azimut-azimut_centre);
        double y=facteur*(Math.cos(hauteur_centre)*Math.sin(hauteur)-Math.sin(hauteur_centre)*Math.cos(hauteur)*Math.cos(azimut-azimut_centre));
        return new Point2D(x, y);
    }
    

   /**
    * methode signe 
    * retourne le signe d'une variable
    * @param x valeur  tester
    * @return le signe de la variable
    */
    public double signe(double x) {
        if (x>0) return 1;
        if (x<0) return -1;
        return 0;
    }
    
}

