package halos;

import geometrie.*;

/**
 * classe Rayon
 * classe modlisant un rayon lumineux
 */
 
public class Rayon {
    private Droite d;   //droite directrice du rayon
    private double I;   //intensit
    

   /**
    * constructeur Rayon 
    * construit un rayon lumineux sans l'initialiser
    */
    public Rayon() {
    
    }

   /**
    * constructeur Rayon  
    * construit un rayon lumineux en spcifiant sa provenance, direction et intensit
    * @param v_ le vecteur d'onde
    * @param P_ le point source
    * @param I_ l'intensit
    */
    public Rayon(Vecteur v_, Point P_, double I_) {
        d = new Droite(v_, P_); I=I_;
    }
    
    /**
    * methode get_v
    * retourne le vecteur d'onde
    * @return le vecteur d'onde
    */
    public Vecteur get_v() {return d.get_v();}

    /**
    * methode get_P
    * retourne le point source
    * @return le point source
    */
    public Point get_P() {return d.get_P();}

    /**
    * methode get_d
    * retourne la droite directrice
    * @return la droite directrice
    */
    public Droite get_d() {return d;}
    
   /**
    * methode get_I
    * retourne l'intensit du rayon lumineux
    * @return l'intensit du rayon lumineux
    */
    public double get_I() {return I;}
    
   /**
    * methode set_v
    * permet de modifier le vecteur d'onde
    * @param v_ le vecteur d'onde
    */
    public void set_v(Vecteur v_) {d.set_v(v_);}

   /**
    * methode set_P
    * permet de modifier le point source
    * @param P_ le point source
    */
    public void set_P(Point P_) {d.set_P(P_);}

   /**
    * methode set_d
    * permet de modifier la droite directrice
    * @param d_ la droite directrice
    */
    public void set_d(Droite d_) {d=d_;}
    
   /**
    * methode set_I
    * permet de modifier l'intensit du rayon lumineux
    * @param I_ l'intensit du rayon lumineux
    */
    public void set_I(double I_) {I=I_;}

   /**
    * methode set 
    * permet de modifier tous les paramtres du rayon lumineux
    * @param v_ le vecteur d'onde
    * @param P_ le point source
    * @param I_ l'intensit
    */
    public void set(Vecteur v_, Point P_, double I_) {d.set(v_, P_); I=I_;}

    /**
    * methode set_P
    * permet de modifier le point source
    * @param x_ la coordonne x
    * @param y_ la coordonne y
    * @param z_ la coordonne z
    */    
    public void set_P(double x_, double y_, double z_) {d.set_P(x_, y_, z_);}

   /**
    * methode affiche
    * affiche les paramtres du rayon lumineux
    */
    public void affiche() {
        System.out.println("Rayon :");
        System.out.println("P : "); d.get_P().affiche();
        System.out.println("v : "); d.get_v().affiche();
        System.out.println("I = "+I);
    }

   /**
    * methode reflect
    * retourne le rayon rflchi sur une forme donne en tenant compte de l'indice de rflexion
    * @param f              la forme gomtrique o le rayon sera rflchi
    * @param coef_reflexion le coefficient de rflexion
    * @return le rayon rflchi
    */
    public Rayon reflect(Forme3D f, double coef_reflexion) {
        Point PI = f.intersect(d);
        Vecteur v = d.get_v(), n = f.get_normale(PI);
        return new Rayon( v.plus( n.fois( -2 * v.scalaire(n) ) ).normalised() , PI, I*coef_reflexion);
    }
    
   /**
    * methode refract
    * retourne le rayon rfract sur une forme donne en tenant compte des indices et coefficients de rfraction
    * @param f                  la forme gomtrique o le rayon sera rflchi
    * @param coef_refraction    le coefficient de rfraction
    * @param n_incident         l'indice optique du milieu d'o arrive le rayon
    * @param n_sortant          l'indice optique du milieu o entre le rayon
    * @return le rayon rfract
    */
    public Rayon refract(Forme3D f, double coef_refraction, double n_incident, double n_sortant) {
        Point PI = f.intersect(d);
        double rapport_n = n_incident / n_sortant;
        Vecteur v = d.get_v(), n = f.get_normale(PI);
    
        double sini = (v.vectoriel(n)).norme();
        double alpha = sini / Math.tan(Math.asin(rapport_n*sini)) * signe(v.scalaire(n)) - v.scalaire(n);
        if (sini==0) alpha=0;
        return new Rayon( v.plus( n.fois( alpha ) ).normalised(), PI, I*coef_refraction );
    }

   /**
    * methode is_refract
    * retourne si le rayon lumineux sera rfract (true) ou subira une rflexion totale (false) sur une forme donne
    * @param f                  la forme gomtrique
    * @param n_incident         l'indice optique du milieu d'o arrive le rayon
    * @param n_sortant          l'indice optique du milieu o entre le rayon
    * @return un boolean indiquant si le rayon lumineux sera rfract (true) ou subira une rflexion totale (false)
    */
    public boolean is_refract(Forme3D f, double n_incident, double n_sortant) {
        Point PI = f.intersect(d);
        double rapport_n = n_incident / n_sortant;
        Vecteur v = d.get_v(), n = f.get_normale(PI);
    
        double sini = (v.vectoriel(n)).norme();
        if(Math.abs(rapport_n*sini)>=1) return false; else return true;
    }
    
   /**
    * methode signe
    * retourne le signe de la variable
    * @param x la variable a tester
    * @return le signe de la variable
    */
    public double signe(double x) {
        if (x>0) return 1;
        if (x<0) return -1;
        return 0;
    }

}
