package halos;

import simu.*;
import java.math.*;
import java.util.Vector;
import geometrie.*;

import halos.*;

import java.awt.image.*;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Color;
import java.lang.String;

/**
 * classe Refraction
 * applet simulant et affichant des interactions de cristaux de glace avec la lumire du soleil
 */
 
public class Refraction extends SimuApplet {
    
   /**
    * methode create_image 
    * cr une image de taille donne reprsentant un cristal et une ligne brise de couleur spcifie  partir de la liste de points fournie en argument
    * Un filtre logarithmique est appliqu et la renormalisation est faite sur les pixels pour lesquels le masque vaut 0
    * @param c               le cristal  afficher
    * @param liste           la liste des points pour la ligne brise
    * @param taille_image    la taille de l'image  crer
    * @param couleur_rayon   la couleur de la ligne brise
    * @return                l'image cre
    */
     Image create_image(Cristal c, Point[] liste, int taille_image, Color couleur_rayon) {
        
        Image img = createImage(taille_image, taille_image);
        Graphics g = (img).getGraphics();
        g.setColor(new Color(0,0,0));
        g.fillRect(0,0,taille_image-1,taille_image-1);

        g.translate(taille_image/2, taille_image/2);
        
        double facteur = -taille_image/(c.get_rayon()*2)*2./3;  //signe - pour rtablir les axes
        int x1, y1, x2, y2;
        
        g.setColor(new Color(0,127,0));
        c.draw(g, facteur);
        g.setColor(couleur_rayon);

        for (int i=0; i<liste.length-1; i++) {
            x1 = (int)Math.floor(liste[i].get_y()*facteur);
            y1 = (int)Math.floor(liste[i].get_z()*facteur);
            x2 = (int)Math.floor(liste[i+1].get_y()*facteur);
            y2 = (int)Math.floor(liste[i+1].get_z()*facteur);
            g.drawLine(x1,y1,x2,y2);
        }
        
        return img;
    }
    

    double dtheta, dphi, dpsi;                                          //les vitesses angulaires
    double theta, phi, psi;                                             //les angles pour le cristal
    double theta0, phi0, psi0;                                          //les angles initiaux du cristal
    double x0, y0;                                                      //le point source du rayon lumineux
    double angle;                                                       //la dviation
    int angle1, angle2;                                                 //la dviation, en degrs et minutes

    Point[] liste;
    
     int taille_image = 512;                                             //taille de la zone d'affichage en pixels
     int type_cristal;                                                   //type de cristal etudi
     double facteur_forme;                                               //facteur de forme du cristal
     int nb_reflexions;                                                  //nombre de rflexions internes supplmentaires imposes
     Vecteur v0 = new Vecteur(0,0,-1);                                   //vecteur du rayon de rfrence pour les calculs de rfraction
     Angles3 angles_c = new Angles3();                                   //angles d'orientation du cristal

     Cristal c;                                                          //Cristal tudi
     Rayon r;                                                            //Rayon test

     double R, R2;                                                       //la rayon et diamtre du cristal tudi

     double[] indices = new double[3];                                   //indices du milieu, pour quelques longueurs d'onde donnes
    int couleur;                                                         //choix de la couleur du rayon lumineux (li au tableau des indices)
    Color[] couleurs = {new Color(255,0,0), new Color(0,255,0), new Color(0,0,255)}; //couleurs d'affichage associes aux indices

   /**
    * constructeur Refraction 
    */
    public Refraction() {
        appletDoc("refraction_doc.xml");
    }
    
   /**
    * methode verifier 
    * verifie les paramtres entrs par l'utilisateur
    */
    public void verifier() {
        
    }
    
   /**
    * methode initCalculLive 
    * initialise l'applet
    * @param in liste des valeurs entres par l'utilisateur
    */
    public void initCalculLive(ListeValeurs in) throws SimuException {

        String str;

        str = in.lireString("typecristal");
        if ("cristal hexagonal".equals(str)) type_cristal = 1;
        if ("cristal pyramidal".equals(str)) type_cristal = 2;
        if ("cristal pyramidal tronqu".equals(str)) type_cristal = 3;
        if ("goutte sphrique".equals(str)) type_cristal = 4;
        if ("cristal bipyramidal".equals(str)) type_cristal = 5;
        facteur_forme = in.lireDouble("facteurforme");

        nb_reflexions = (int)in.lireDouble("reflexions");
        str = in.lireString("materiau");
        if ("glace".equals(str)) {
            indices[0] = 1.308;
            indices[1] = 1.313;
            indices[2] = 1.316;
        }
        if ("eau".equals(str)) {
            indices[0] = 1.3318;
            indices[1] = 1.3360;
            indices[2] = 1.3417;
        }

        str = in.lireString("couleur");
        if ("Rouge".equals(str)) couleur=0;
        if ("Vert".equals(str)) couleur=1;
        if ("Bleu".equals(str)) couleur=2;

        dtheta = in.lireDouble("vtheta")*Math.PI/180;
        dphi = in.lireDouble("vphi")*Math.PI/180;
        dpsi = in.lireDouble("vpsi")*Math.PI/180;

        theta0 = in.lireDouble("theta0")*Math.PI/180;
        phi0 = in.lireDouble("phi0")*Math.PI/180;
        psi0 = in.lireDouble("psi0")*Math.PI/180;

        x0 = in.lireDouble("x0");
        y0 = in.lireDouble("y0");


        verifier(); //vrifie la validit des paramtres entrs

        //cration des objets, orientation du cristal
        theta=0; phi=0; psi=0;
        c = new Cristal(type_cristal, facteur_forme, indices[couleur]);
        R = c.get_rayon(); R2=R*2;
        r = new Rayon(v0, new Point(x0,y0,R2), 1);
        
        c.oriente(new Angles3(theta0,phi0,psi0));
        c.set_reference();

        //calcul de l'intraction rayon/cristal
        liste = c.trajectoire(r, 0, nb_reflexions);
        Vecteur v = c.vecteur_sortie(r, 0, nb_reflexions);

        //conversion de la dviation en degrs minutes
        angle = Math.acos(v.scalaire(v0))*180/Math.PI;
        angle1 = (int)Math.floor(angle);
        angle2 = (int)Math.floor((angle-angle1)*60);

        if (liste.length<2) {liste = new Point[2]; liste[0]=new Point(x0,y0,R2); liste[1]=new Point(x0,y0,-R2);} //rayon rectiligne si pas d'interception
    }
    
   /**
    * methode calculLive 
    * effectue le calcul d'un certain nombre de rayons lumineux avant de retourner le rsultat
    * @return l'image et l'histogramme du halo
    */
    public ListeValeurs calculLive() throws SimuException {
    
        ListeValeurs out = new ListeValeurs();
 
        //rotation du rayon
        for (int i=0; i<liste.length; i++) {
            liste[i].rotated_z(psi);
            liste[i].rotated_y(theta);
            liste[i].rotated_z(phi);
        }
        
        Image img = create_image(c, liste, taille_image, couleurs[couleur]);   //cration de l'image
        
        //retour  la direction originale du rayon
        for (int i=0; i<liste.length; i++) {
            liste[i].rotated_z(-phi);
            liste[i].rotated_y(-theta);
            liste[i].rotated_z(-psi);
        }
        
        c.tourne(dtheta,dphi,dpsi); //rotation du cristal

        theta+=dtheta; phi+=dphi; psi+=dpsi;

        //envoi des donnes  simulab
        out.ajouter("image", img);
        out.ajouterString("angle", "Dviation : "+angle1+""+angle2+"'");
        return out;
    }
    

   /**
    * methode signe 
    * retourne le signe d'une variable
    * @param x valeur  tester
    * @return le signe de la variable
    */
    public double signe(double x) {
        if (x>0) return 1;
        if (x<0) return -1;
        return 0;
    }
    
}


